/*
 * TLC5955.C
 *
 *  Created on: Aug 17, 2017
 *      Author: x0268118
 */
#include <ti/devices/msp432p4xx/driverlib/driverlib.h>
#include "ti/devices/msp432p4xx/inc/msp.h"
/* Standard Includes */
#include <stdint.h>
#include <stdbool.h>
void error(void);
void Init_Clock()
{
    volatile uint32_t i;
    uint32_t currentPowerState;

    WDT_A->CTL = WDT_A_CTL_PW |             // Stop WDT
                 WDT_A_CTL_HOLD;

    P1->DIR |= BIT0;

    /* NOTE: This example assumes the default power state is AM0_LDO.
     * Refer to  msp432p401x_pcm_0x code examples for more complete PCM operations
     * to exercise various power state transitions between active modes.
     */

    /* Step 1: Transition to VCORE Level 1: AM0_LDO --> AM1_LDO */
    /* Get current power state, if it's not AM0_LDO, error out */
    currentPowerState = PCM->CTL0 & PCM_CTL0_CPM_MASK;
    if (currentPowerState != PCM_CTL0_CPM_0)
        error();

    while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
    PCM->CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_AMR_1;
    while ((PCM->CTL1 & PCM_CTL1_PMR_BUSY));
    if (PCM->IFG & PCM_IFG_AM_INVALID_TR_IFG)
        error();                            // Error if transition was not successful
    if ((PCM->CTL0 & PCM_CTL0_CPM_MASK) != PCM_CTL0_CPM_1)
        error();                            // Error if device is not in AM1_LDO mode

    /* Step 2: Configure Flash wait-state to 1 for both banks 0 & 1 */
    FLCTL->BANK0_RDCTL = (FLCTL->BANK0_RDCTL & ~(FLCTL_BANK0_RDCTL_WAIT_MASK)) |
            FLCTL_BANK0_RDCTL_WAIT_1;
    FLCTL->BANK1_RDCTL = (FLCTL->BANK0_RDCTL & ~(FLCTL_BANK1_RDCTL_WAIT_MASK)) |
            FLCTL_BANK1_RDCTL_WAIT_1 ;

    /* Step 3: Configure HFXT to use 48MHz crystal, source to MCLK & HSMCLK*/


    PJ->SEL0 |= BIT2 | BIT3;                // Configure PJ.2/3 for HFXT function
    PJ->SEL1 &= ~(BIT2 | BIT3);

    CS->KEY = CS_KEY_VAL ;                  // Unlock CS module for register access
    CS->CTL2 |= CS_CTL2_HFXT_EN | CS_CTL2_HFXTFREQ_6 | CS_CTL2_HFXTDRIVE;
    while(CS->IFG & CS_IFG_HFXTIFG)
        CS->CLRIFG |= CS_CLRIFG_CLR_HFXTIFG;

    /* Select MCLK & HSMCLK = HFXT, no divider */
    CS->CTL1 = CS->CTL1 & ~(CS_CTL1_SELM_MASK | CS_CTL1_DIVM_MASK | CS_CTL1_SELS_MASK | CS_CTL1_DIVHS_MASK) |
            CS_CTL1_SELM__HFXTCLK | CS_CTL1_SELS__HFXTCLK;

    CS->KEY = 0;                            // Lock CS module from unintended accesses

    }

void Init_TLC5957() // initialize PIN
     {
      GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN5|GPIO_PIN6|GPIO_PIN7); //P4.5 SIN P4.6 SCLK P4.7 XLAT
      GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN5|GPIO_PIN6|GPIO_PIN7);
      GPIO_setAsInputPin(GPIO_PORT_P6, GPIO_PIN5);   //P6.5 SOUT
      //GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P4,GPIO_PIN3,GPIO_PRIMARY_MODULE_FUNCTION);// P4.3->MCLK   for GSCLK
    }

void SIN(bool i)  //i only 1 or 0
{
    if(i==0)  GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN5);
    else   GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN5);
}
void SCLK(bool i) //i only 1 or 0
{
    if(i==0)  GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN6);
        else   GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN6);
}

void LAT(bool i) //i only 1 or 0
{
    if(i==0)  GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN7);
        else   GPIO_setOutputHighOnPin(GPIO_PORT_P4, GPIO_PIN7);
}
void GCLK(bool i) //i only 1 or 0
{
    if(i==0)
    {
        GPIO_setAsOutputPin(GPIO_PORT_P4, GPIO_PIN3);
        GPIO_setOutputLowOnPin(GPIO_PORT_P4, GPIO_PIN3);
    }
    else  GPIO_setAsPeripheralModuleFunctionOutputPin(GPIO_PORT_P4, GPIO_PIN3,GPIO_PRIMARY_MODULE_FUNCTION);
}

/*FC Data Write Function*/
void WriteFCData(char LGSE2,bool Poker,char BC,int CCR,int CCG, int CCB,char LGSE1, char SEL_SCK_EDGE,char LGSE3,char ESPWM, char SEL_PCHG,char SEL_GCK_EDGE,
                 char XREFRESH, char SEL_GDLY, char SEL_TD0, char LODVTH)
{
    int k;
    char temp1,temp2;
    int temp3,temp4;
   LAT(0);LAT(1);
   for(k=14;k>=0;k--)  //FCWRTEN
   {
    SIN(1);// SIN don't care
    SCLK(0);
   __delay_cycles(1); //1/25M=40ns
    SCLK(1);
   }
   LAT(0);
 // FC Data
  for(k=47;k>=45;k--)
  {
      temp1=1<<(k-45);
      temp2=LGSE2&temp1;
      if(temp2==temp1)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
  }
  // Poker trans mode  44th common shift register
  if(Poker==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //BC Data
  for(k=43;k>=41;k--)
  {
      temp1=1<<(k-41);
      temp2=BC&temp1;
      if(temp2==temp1)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
  }
  //  CCR Data
  for(k=40;k>=32;k--)
  {
      temp3=1<<(k-32);
      temp4=CCR&temp3;
      if(temp4==temp3)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
     }
  //  CCG Data
  for(k=31;k>=23;k--)
  {
      temp3=1<<(k-23);
      temp4=CCG&temp3;
      if(temp4==temp3)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
     }
  //  CCB Data
  for(k=22;k>=14;k--)
  {
      temp3=1<<(k-14);
      temp4=CCB&temp3;
      if(temp4==temp3)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
     }
  //LGSE1 Data
  for(k=13;k>=11;k--)
  {
      temp1=1<<(k-11);
      temp2=LGSE1&temp1;
      if(temp2==temp1)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
  }
  //SEL_SCK_EDGE
  if(SEL_SCK_EDGE==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //LGSE3
  if(LGSE3==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //ESPWM
  if(ESPWM==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //SEL_PCHG
  if(SEL_PCHG==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //SEL_GCK_EDGE
  if(SEL_GCK_EDGE==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //XREFRESH
  if(XREFRESH==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
/// WRTFC Command
  LAT(1);
  //SEL_GDLY
  if(SEL_GDLY==0)
      SIN(0);
  else
      SIN(1);
  SCLK(0);
  __delay_cycles(1); //1/25M=40ns
  SCLK(1);
  //SEL_TD0
  for(k=3;k>=2;k--)
  {
      temp1=1<<(k-2);
      temp2=SEL_TD0&temp1;
      if(temp2==temp1)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
  }
  //LODVTH
  for(k=1;k>=0;k--)
  {
      temp1=1<<k;
      temp2=LODVTH&temp1;
      if(temp2==temp1)
          SIN(1);
      else
          SIN(0);
      SCLK(0);
      __delay_cycles(1); //1/25M=40ns
      SCLK(1);
  }
  /// WRTFC Command
  LAT(0);
}

int i,j,k,temp1,temp2;
uint16_t bit5=~BIT5;
uint16_t bit6=~BIT6;
uint16_t bit7=~BIT7;

/*GS Data Write Function - Normal Mode - GPIO Driving*/
void  WriteGSData(int *temp)
{
    P4OUT = P4OUT & bit7;// Latch Low
    for (i=15;i>=0;i--)
    {
        for(j=47;j>=0;j--)
        {
         temp1=1<<(j%16);
         temp2=temp[i*3+j/16]&temp1;
         if(temp2==temp1)
         P4OUT = P4OUT|BIT5;// SIN High
         else
         P4OUT = P4OUT&bit5;// SIN Low

         P4OUT = P4OUT&bit6;// SCLK Low
         P4OUT = P4OUT|BIT6;// SCLK High

         k=i*3+j/16;
         if(j==3&&k==0) //LATGS Command
             P4OUT = P4OUT|BIT7;// Latch High
         else if (j==1)   //WRTGS Command
             P4OUT = P4OUT|BIT7;// Latch High
         }
        P4OUT = P4OUT & bit7;// Latch Low
    }
}

void  WriteGSData_Lastline(int *temp)
{
     int i,j,k,temp1,temp2;
     LAT(0);
    for (i=15;i>=0;i--)
    {
        for(j=47;j>=0;j--)
        {
         temp1=1<<(j%16);
         temp2=temp[i*3+j/16]&temp1;
         if(temp2==temp1)
             SIN(1);
         else
             SIN(0);
         SCLK(0);
         //__delay_cycles(1); //6/25M=240ns
         SCLK(1);
         k=i*3+j/16;
         if(j==7&&k==0) //LINERESET Command
             LAT(1);
         else if (j==1)   //WRTGS Command
             LAT(1);
         }
        LAT(0);
    }
}

/*GS Data Write Function - Poker Mode - GPIO Driving*/
void  WriteGSData_Poker(int *temp)
{
    P4OUT = P4OUT & bit7;// Latch Low
    for (i=8;i>=0;i--)
    {
        for(j=47;j>=0;j--)
        {
         //temp1=1<<i;
         //temp2=temp[j]&temp1;
         if((temp[j]&(1<<i))==(1<<i))
         P4OUT = P4OUT|BIT5;// SIN High
         else
         P4OUT = P4OUT&bit5;// SIN Low

         P4OUT = P4OUT&bit6;// SCLK Low
         P4OUT = P4OUT|BIT6;// SCLK High


         if(j==3&&i==0) //LATGS Command
             P4OUT = P4OUT|BIT7;// Latch High
         else if (j==1)   //WRTGS Command
             P4OUT = P4OUT|BIT7;// Latch High
         }
        if(i==0)
        {
            GPIO_setOutputLowOnPin(GPIO_PORT_P2,GPIO_PIN3|GPIO_PIN4|GPIO_PIN6|GPIO_PIN7);
            GPIO_setOutputLowOnPin(GPIO_PORT_P3,GPIO_PIN5|GPIO_PIN7);
            GPIO_setOutputLowOnPin(GPIO_PORT_P5,GPIO_PIN1|GPIO_PIN6);
            GPIO_setOutputLowOnPin(GPIO_PORT_P6,GPIO_PIN6|GPIO_PIN7);
        }
        P4OUT = P4OUT & bit7;// Latch Low
    }
}

void  WriteGSData_Poker_Lastline(int *temp)
{
     int i,j,temp1,temp2;
     LAT(0);
    for (i=7;i>=0;i--)
    {
        for(j=47;j>=0;j--)
        {
         temp1=1<<i;
         temp2=temp[j]&temp1;
         if(temp2==temp1)
             SIN(1);
         else
             SIN(0);
         SCLK(0);
         SCLK(1);
         if(j==7&&i==0) //LATGS Command
             LAT(1);
         else if (j==1)   //WRTGS Command
             LAT(1);

         }

       // __delay_cycles(1); //6/25M=240ns
        LAT(0);
    }
}

void error(void)
{
    volatile uint32_t i;

    while (1)
    {
        P1->OUT ^= BIT0;
        for(i = 20000; i> 0; i--);          // Blink LED forever
    }
}
